﻿using Microsoft.EntityFrameworkCore;
using UserMicroservice.Models;

namespace UserMicroservice.Services
{
    public class UserService : IUserService
    {
        private readonly ApplicationDbContext _context;

        public UserService(ApplicationDbContext context)
        {
            _context = context;
        }

        public bool DeleteUser(string username)
        {
            User? user = _context.Users.SingleOrDefault(user => user.Username == username);
            if(user == null)
                return false;

            _context.Users.Remove(user);
            _context.SaveChanges();

            return true;
        }

        public List<User> GetUsers() 
        { 
            List<User> users = _context.Users.ToList();
            return users;
        }

        public User? GetUser(string username)
        {
            User? user = _context.Users.SingleOrDefault(user => user.Username == username);
            return user;
        }

        public User? GetUser(int userId)
        {
            User? user = _context.Users.SingleOrDefault(user=> user.Id ==  userId);
            return user;
        }

        public User? GetUser(string username, string password)
        {
            User? user = _context.Users
                .SingleOrDefault(user => user.Username == username && user.PasswordHash == password);
            return user;
        }

        public User CreateUser(string email, string userName, string password)
        {
            User user = new User(userName, email, password);
            _context.Users.Add(user);
            _context.SaveChanges();

            return user;            
        }

        public User UpdateUser(User updatedUser)
        {
            _context.Users
                        .Where(user => user.Id == updatedUser.Id)
                        .ExecuteUpdate(setters => setters
                            .SetProperty(user => user.Username, updatedUser.Username)
                            .SetProperty(user => user.PasswordHash, updatedUser.PasswordHash));

            _context.SaveChanges();

            return updatedUser;
        }
    }
}
